--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library work;
use work.sha3_pkg.all;
use work.sha2_pkg.all;
use work.ntru_pkg.all;


entity b2t is
	generic ( w	: integer := 32);
	port (
		input   : in std_logic_vector(3*w-1 downto 0);
		output   : out  std_logic_vector(4*w-1 downto 0)
	);
end b2t;

architecture arch of b2t is

	type bits_2_trits_rom is array(0 to 7) of std_logic_vector(1 downto 0);
	type val_3i_array is array(0 to w-1) of std_logic_vector(1 downto 0);
	type M_buf_3i_array is array(0 to w-1) of std_logic_vector(2 downto 0);
	type comb_val_array is array(0 to w-1) of std_logic_vector(3 downto 0);

	signal M_buf_3i				: M_buf_3i_array;
	signal val1_3i, val2_3i		: val_3i_array;
	signal comb_val_12			: comb_val_array;

	-- constants
	constant bits_2_trits1		: bits_2_trits_rom := ("00", "00", "00", "01", "01", "01", "10", "10");
	constant bits_2_trits2		: bits_2_trits_rom := ("00", "01", "10", "00", "01", "10", "00", "01");
	constant Ones 				: std_logic_vector(w-1 downto 0) := (others => '1');

begin

	ii:  for i in 0 to w-1 generate
		M_buf_3i(i) 	<= input((3*i)+2 downto 3*i);
		val1_3i(i) 		<= bits_2_trits1(conv_integer(M_buf_3i(i)));
		val2_3i(i) 		<= bits_2_trits2(conv_integer(M_buf_3i(i)));
		comb_val_12(i)	<= val1_3i(i) & val2_3i(i);
		output((4*i)+3 downto 4*i) <= comb_val_12(i);
	end generate;

end arch;

