-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity check1_rom is
	generic
	(
		ADDR_WIDTH : integer := 6;
		DATA_WIDTH : integer := 2
	);
	port
	(
		addr  : in std_logic_vector(ADDR_WIDTH-1 downto 0);
		dout_num1 : out std_logic_vector(DATA_WIDTH-1 downto 0);
        dout_num2 : out std_logic_vector(DATA_WIDTH-1 downto 0)
	);
end check1_rom;

architecture arch of check1_rom is

	type mem is array (0 to 2**ADDR_WIDTH-1) of std_logic_vector(DATA_WIDTH-1 downto 0);

	constant num1_rom : mem :=
	(
        0 => "00",
        1 => "01",
        2 => "00",
        3 => "00",
        4 => "01",
        5 => "10",
        6 => "01",
        7 => "01",
        8 => "00",
        9 => "01",
        10 => "00",
        11 => "00",
        12 => "00",
        13 => "01",
        14 => "00",
        15 => "00",
        16 => "01",
        17 => "10",
        18 => "01",
        19 => "01",
        20 => "10",
        21 => "11",
        22 => "10",
        23 => "10",
        24 => "01",
        25 => "10",
        26 => "01",
        27 => "01",
        28 => "01",
        29 => "10",
        30 => "01",
        31 => "01",
        32 => "00",
        33 => "01",
        34 => "00",
        35 => "00",
        36 => "01",
        37 => "10",
        38 => "01",
        39 => "01",
        40 => "00",
        41 => "01",
        42 => "00",
        43 => "00",
        44 => "00",
        45 => "01",
        46 => "00",
        47 => "00",
        48 => "00",
        49 => "01",
        50 => "00",
        51 => "00",
        52 => "01",
        53 => "10",
        54 => "01",
        55 => "01",
        56 => "00",
        57 => "01",
        58 => "00",
        59 => "00",
        60 => "00",
        61 => "01",
        62 => "00",
        63 => "00"
	);

    constant num2_rom : mem :=
    (
        0 => "00",
        1 => "00",
        2 => "01",
        3 => "00",
        4 => "00",
        5 => "00",
        6 => "01",
        7 => "00",
        8 => "01",
        9 => "01",
        10 => "10",
        11 => "01",
        12 => "00",
        13 => "00",
        14 => "01",
        15 => "00",
        16 => "00",
        17 => "00",
        18 => "01",
        19 => "00",
        20 => "00",
        21 => "00",
        22 => "01",
        23 => "00",
        24 => "01",
        25 => "01",
        26 => "10",
        27 => "01",
        28 => "00",
        29 => "00",
        30 => "01",
        31 => "00",
        32 => "01",
        33 => "01",
        34 => "10",
        35 => "01",
        36 => "01",
        37 => "01",
        38 => "10",
        39 => "01",
        40 => "10",
        41 => "10",
        42 => "11",
        43 => "10",
        44 => "01",
        45 => "01",
        46 => "10",
        47 => "01",
        48 => "00",
        49 => "00",
        50 => "01",
        51 => "00",
        52 => "00",
        53 => "00",
        54 => "01",
        55 => "00",
        56 => "01",
        57 => "01",
        58 => "10",
        59 => "01",
        60 => "00",
        61 => "00",
        62 => "01",
        63 => "00"
    );

begin

	dout_num1 <= num1_rom(conv_integer(addr));
    dout_num2 <= num2_rom(conv_integer(addr));
end arch;
