-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.ntru_pkg.all;

entity check_1 is
    generic(
        iw : integer := 32;
        ow : integer := 96
    );
    Port (
           clk           : in std_logic;
           rst           : in std_logic;
           en_check1_reg : in std_logic;
           mp_ci         : in std_logic_vector(127 downto 0);
           check1_last   : in std_logic;
           check_1       : out std_logic
           );
end check_1;

architecture arch of check_1 is
    --DEBUG
    -- type debug_array is array (0 to 63) of std_logic_vector(1 downto 0);
    -- signal debug_input : debug_array;
    --DEBUG

    type num_array is array (0 to 21) of std_logic_vector(1 downto 0);
    signal num1_array, num2_array : num_array;

    type btree_l1_array is array (0 to 10) of std_logic_vector(2 downto 0);
    type btree_l2_array is array (0 to 5) of std_logic_vector(3 downto 0);
    type btree_l3_array is array (0 to 2) of std_logic_vector(4 downto 0);
    type btree_l4_array is array (0 to 1) of std_logic_vector(5 downto 0);

    signal btree_l1_1, btree_l1_2 : btree_l1_array;
    signal btree_l2_1, btree_l2_2 : btree_l2_array;
    signal btree_l3_1, btree_l3_2 : btree_l3_array;
    signal btree_l4_1, btree_l4_2 : btree_l4_array;
    signal btree_l5_1, btree_l5_2, btree_l5_0 : std_logic_vector(6 downto 0);
    signal reg_num1_out, reg_num2_out, reg_num0_out, reg_num1_in, reg_num2_in,
    reg_num0_in : std_logic_vector(30 downto 0);
    signal check_num1_valid, check_num2_valid, check_num0_valid : std_logic;
    signal CHECK1_MinMsgRep_WT : integer;
    signal Last_Pad            : integer;

begin
    Param_1087: if PARAM_N = 1087 generate
        CHECK1_MinMsgRep_WT   <= CHECK1_MinMsgRep_WT_1087;
        Last_Pad              <= Last_Pad_1087;
    end generate;
    Param_1499: if PARAM_N = 1499 generate
        CHECK1_MinMsgRep_WT   <= CHECK1_MinMsgRep_WT_1499;
        Last_Pad              <= Last_Pad_1499;
    end generate;

    --DEBUG
    -- GEN_0: for i in 0 to 63 generate
    --     debug_input(i) <= mp_ci(2*(i+1)-1 downto 2*i);
    -- end generate;
    --DEBUG

    GEN_1: for i in 0 to 20 generate
        Rom_isnt: entity work.check1_rom(arch)
            port map(
                addr        => mp_ci(6*(i+1)-1 downto 6*i),
                dout_num1   => num1_array(i),
                dout_num2   => num2_array(i)
            );

    end generate;
        num1_array(21) <= '0' & mp_ci(126);
        num2_array(21) <= '0' & mp_ci(127);

    GEN_BTREE_L1: for i in 0 to 10 generate
        btree_l1_1(i) <= ('0' & num1_array(2*i))+('0' & num1_array((2*i)+1));
        btree_l1_2(i) <= ('0' & num2_array(2*i))+('0' & num2_array((2*i)+1));
    end generate;

    GEN_BTREE_L2: for i in 0 to 4 generate
        btree_l2_1(i) <= ('0' & btree_l1_1(2*i))+('0' & btree_l1_1((2*i)+1));
        btree_l2_2(i) <= ('0' & btree_l1_2(2*i))+('0' & btree_l1_2((2*i)+1));
    end generate;
    btree_l2_1(5) <= '0' & btree_l1_1(10);
    btree_l2_2(5) <= '0' & btree_l1_2(10);

    GEN_BTREE_L3: for i in 0 to 2 generate
        btree_l3_1(i) <= ('0' & btree_l2_1(2*i))+('0' & btree_l2_1((2*i)+1));
        btree_l3_2(i) <= ('0' & btree_l2_2(2*i))+('0' & btree_l2_2((2*i)+1));
    end generate;

    btree_l4_1(0) <= ('0' & btree_l3_1(0))+('0' & btree_l3_1(1));
    btree_l4_2(0) <= ('0' & btree_l3_2(0))+('0' & btree_l3_2(1));
    btree_l4_1(1) <= '0' & btree_l3_1(2);
    btree_l4_2(1) <= '0' & btree_l3_2(2);

    btree_l5_1 <= ('0' & btree_l4_1(0)) + ('0' & btree_l4_1(1));
    btree_l5_2 <= ('0' & btree_l4_2(0)) + ('0' & btree_l4_2(1));

    btree_l5_0 <= (64 - btree_l5_1 - btree_l5_2 - Last_Pad) when
        check1_last = '1' else (64 - btree_l5_1 - btree_l5_2);

    reg_num1_in <= (x"000000" & btree_l5_1) + reg_num1_out;
    REG_NUM1 : entity work.regn generic map ( N => 31, init => zeros(30 downto 0))
    port map ( clk => clk, rst => rst, en => en_check1_reg, input =>reg_num1_in,
    output => reg_num1_out);

    reg_num2_in <= (x"000000" & btree_l5_2) + reg_num2_out;
    REG_NUM2 : entity work.regn generic map ( N => 31, init => zeros(30 downto 0))
    port map ( clk => clk, rst => rst, en => en_check1_reg, input =>reg_num2_in,
    output => reg_num2_out);

    reg_num0_in <= (x"000000" & btree_l5_0) + reg_num0_out;
    REG_NUM0 : entity work.regn generic map ( N => 31, init => zeros(30 downto 0))
    port map ( clk => clk, rst => rst, en => en_check1_reg, input =>reg_num0_in,
    output => reg_num0_out);

    check_num1_valid <= '1' when (reg_num1_out >= CHECK1_MinMsgRep_WT) else '0';
    check_num2_valid <= '1' when (reg_num2_out >= CHECK1_MinMsgRep_WT) else '0';
    check_num0_valid <= '1' when (reg_num0_out >= CHECK1_MinMsgRep_WT) else '0';
    check_1 <= check_num1_valid and check_num2_valid and check_num0_valid;

end arch;
