-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library work;
use work.ntru_pkg.all;

--! Data Forming Unit
entity dfu is
generic
(
	w : integer := 32
);
port
(
	-- inputs
	clk 	: in std_logic;
	rst 	: in std_logic;
	a       : in std_logic_vector(w-1 downto 0);
	b 		: in std_logic_vector(w-1 downto 0);
	c       : in std_logic_vector(w-1 downto 0);
    d       : in std_logic_vector(w-1 downto 0);

	-- control signals
    en_reg  : in std_logic;
    en_backup_reg : in std_logic;
    sel_backup    : in std_logic;
    sel_i 	: in std_logic_vector(1 downto 0);
	sel_oid : in std_logic;
	sel_o 	: in std_logic_vector(2 downto 0);

	-- outputs
	r 		: out std_logic_vector(w-1 downto 0)
);
end dfu;

-- ==========================================================================
architecture arch of dfu is

	constant ZEROs : std_logic_vector(w-1 downto 0) := (others => '0');
    signal OID : std_logic_vector(23 downto 0);

	signal r0            : std_logic_vector(w-1 downto 0);
    signal r1, r1_backup, from_regin_mux : std_logic_vector(23  downto 0);
	signal from_mux      : std_logic_vector(w-1 downto 0);
begin

    PARAM_GEN_1087: if PARAM_N = 1087 generate
        OID <= OID_1087;
    end generate;

    PARAM_GEN_1499: if PARAM_N = 1499 generate
        OID <= OID_1499;
    end generate;

	with sel_i select
		r0 <= 	a when "00",
				b when "01",
				c when "10",
                d when others;

    from_regin_mux <= r1_backup when(sel_backup = '1') else r0(23 downto 0);

	Orig_reg : entity work.regn(struct)
        generic map(
            N => 24,
            init => ZEROs(23 downto 0)
            )
		port map(
            clk    => clk,
            rst    => rst,
            en     => en_reg,
            input  => from_regin_mux,
            output => r1
            );

    Backup_reg : entity work.regn(struct)
        generic map(
            N => 24,
            init => ZEROs(23 downto 0)
            )
		port map(
            clk    => clk,
            rst    => rst,
            en     => en_backup_reg,
            input  => r1,
            output => r1_backup
            );

	from_mux <= OID & r0(31 downto 24) when (sel_oid = '1') else r1 & r0(31 downto 24);

	with sel_o select
		r <= 	from_mux when "000",                             --! 24-bits remaining
				(r1(23 downto 8) & r0(31 downto 16)) when "001", --! 16-bits remaining msg
				(r1(23 downto 16) & r0(31 downto 8)) when "010", --! 8-bits remaining  msg
                (r1(15 downto 0) & r0(31 downto 16)) when "011", --! 16-bits remaining b
                (r1(7 downto 0) & r0(31 downto 8)) when "100",   --! 8-bits remaining b
				r0 when others;                                  --! No bits remaining

end arch;

-- ==========================================================================
