-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand, Kevin Briggs
-- =============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity dfu_m is
    generic(
        iw : integer := 32;
        ow : integer := 96
    );
    Port (
           clk : in std_logic;
           rst : in std_logic;

           sel_reg_oct  : in std_logic;
           sel_data_sig : in std_logic;
           sel_sipo_in : in std_logic;
           en_mreg : std_logic;
           en_sipo : std_logic;

           m : in STD_LOGIC_VECTOR (31 downto 0);
           b : in STD_LOGIC_VECTOR (31 downto 0);
           octl : in STD_LOGIC_VECTOR (7 downto 0);
           Mbin : out STD_LOGIC_VECTOR (95 downto 0));
end dfu_m;

architecture Behavioral of dfu_m is

    constant zeros     : std_logic_vector(7 downto 0) := (others => '0');

    signal mreg_out    : std_logic_vector(7 downto 0);
    signal reg_oct_sig : std_logic_vector(7 downto 0);
    signal msig     : std_logic_vector(iw-1 downto 0);
    signal data_sig : std_logic_vector(iw-1 downto 0);
    signal sipo_in : std_logic_vector(iw-1 downto 0);


begin
    mreg : entity work.regn generic map ( N => 8, init => zeros)
    port map ( clk => clk, rst => rst, en => en_mreg, input => m(7 downto 0), output => mreg_out );

    sipo : entity work.sipo generic map ( N => ow, M => iw)
    port map ( clk => clk, rst => rst, en => en_sipo, input => sipo_in, output => Mbin );

    with sel_sipo_in select sipo_in <=
        (others => '0') when '0',
        data_sig when others;

    with sel_data_sig select data_sig <=
        msig when '0',
        b when others;

    msig <= reg_oct_sig & m(31 downto 8);

    with sel_reg_oct select reg_oct_sig <=
        mreg_out when '0',
        octl when others;

end Behavioral;
