-- =====================================================================
-- Copyright � 2018 by Cryptographic Engineering Research Group (CERG)
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Malik Umar Sharif, Kevin Briggs
-- =====================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;

-- Input Data Conversion Unit for encryption / decryption
entity idcu_ed is
generic
(
	iw : integer := 32;
	ow_h_e : integer := 22;
	ow_h_ram : integer := 32
);
port
(
    clk : in std_logic;
    rst : in std_logic;

    en_sipo_h_e    : in std_logic;
    en_sipo_h_ram  : in std_logic;
    en_piso_h_ram  : in std_logic;
    ld_piso_h_ram  : in std_logic;

    pdi_data : in std_logic_vector(iw-1 downto 0);
    h_e      : out std_logic_vector(ow_h_e -1 downto 0);
    h_ram    : out std_logic_vector(ow_h_ram-1 downto 0)
);
end idcu_ed;

-- ==========================================================================

architecture arch of idcu_ed is

    signal h_ram_sig : std_logic_vector(263 downto 0);
    signal sipo_out  : std_logic_vector(ow_h_e-11-1 downto 0);

begin
    --! SIPO should be used for input bigger than 22
    -- sipo_h_e : entity work.sipo generic map ( M => 11, N=> (ow_h_e-11))
    -- port map ( clk => clk, rst => rst, en => en_sipo_h_e, input => pdi_data(10 downto 0), output => sipo_out);
    reg_h: entity work.regn(struct) generic map (N => 11,  init => zeros(10 downto 0))
        port map (clk => clk, rst => rst, en => en_sipo_h_e, input => pdi_data(10 downto 0), output => sipo_out);

    -- h_e <= sipo_out & pdi_data(10 downto 0);
    h_e <= pdi_data(10 downto 0) & sipo_out;

    sipo_h_ram : entity work.sipo generic map ( M => 11, N=> 264)
        port map ( clk => clk, rst => rst, en => en_sipo_h_ram, input => pdi_data(10 downto 0), output => h_ram_sig );

    piso_h_ram : entity work.piso generic map ( N => 256, M=> ow_h_ram)
        port map ( clk => clk, rst => rst, en => en_piso_h_ram, ld => ld_piso_h_ram, input => h_ram_sig(263 downto 8), output => h_ram );


end arch;
-- ==========================================================================
