-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library work;
use work.ntru_pkg.all;

entity ntru_top is
    port(
        clk             : in  std_logic;
        rst             : in  std_logic;

        sdi_valid       : in std_logic;
        sdi_ready       : out std_logic;
        sdi_data        : in std_logic_vector(SW-1 downto 0);

        pdi_valid       : in std_logic;
        pdi_ready       : out std_logic;
        pdi_data        : in std_logic_vector(PW-1 downto 0);

        rdi_valid       : in std_logic;
        rdi_ready       : out std_logic;
        rdi_data        : in std_logic_vector(PW-1 downto 0);

        do_valid        : out std_logic;
        do_ready        : in std_logic;
        do_data         : out std_logic_vector(PW-1 downto 0)
    );

end ntru_top;


architecture structure of ntru_top is

signal R_fe_valid_to_ctrl, Rfe_rd_from_ctrl, sel_Rfe_rd, he_fifo_read,
    he_fifo_empty, mask_ready, mask_valid, ce_valid, ce_read, bpgm_mgf_start,
    bpgm_mgf_done, bpgm_mgf_sel, bpgm_mgf_rst, chk_failed_bwc1, chk_failed_bwc2,
    decrypt, sel_param, check_3, check1_rst, check1_en_reg, check_1, check1_last,
    poly_mult_sdi_ready, poly_mult_start, poly_mult_done, poly_mult_update_h,
    poly_mult_rF_sel, sel_r_F, sel_m_cm_di, sel_b_cb_di, sel_hTrunc_addr,
    sel_odcu, sel_ci_mpr, sel_m_cm_addr, sel_b_cb_addr, hTrunc_we, sel_F_addr,
    F_we, m_cm_we, m_cm_rd_ctrl, b_cb_we, dfum_sel_regoct, dfum_sel_data,
    dfum_sel_sipo, dfum_en_mreg, dfum_en_sipo, odcue_di_valid, odcue_di_ready,
    odcud_en_piso, odcud_ld_piso, odcud_en_mreg, odcud_sel_cm, odcud_sel_cb,
    idcued_en_sipo_he, idcued_en_sipo_hram, idcued_en_piso_hram,
    idcued_ld_piso_hram, bpgm_mgf_Rferd, sel_ctrl_header, ctrl_do_valid,
    ctrl_do_ready, poly_mult_rst,
    error

    : std_logic;

signal msg_len, F_addr_ctrl, odcud_coctL
    : std_logic_vector(7 downto 0);

signal hTrunc_addr_ctrl, b_cb_addr_ctrl
    : std_logic_vector(2 downto 0);

signal odcud_check2
    : std_logic_vector(3 downto 0);

signal m_cm_addr_ctrl
    : std_logic_vector(5 downto 0);

signal ctrl_dout
    : std_logic_vector(31 downto 0);


begin

    top_datapath_inst: entity work.top_datapath(structure)
    port map(
        clk       => clk,
        rst       => rst,

        sdi_data  => sdi_data,
        pdi_data  => pdi_data,
        rdi_data  => rdi_data,
        do_data   => do_data,

        ctrl_dout => ctrl_dout,
        sel_ctrl_header => sel_ctrl_header,
        ctrl_do_valid   => ctrl_do_valid,
        ctrl_do_ready   => ctrl_do_ready,

        R_fe_valid_to_ctrl => R_fe_valid_to_ctrl,
        Rfe_rd_from_ctrl   => Rfe_rd_from_ctrl,
        sel_Rfe_rd         => sel_Rfe_rd,

        he_fifo_read    => he_fifo_read,
        he_fifo_empty   => he_fifo_empty,

        mask_ready      => mask_ready,
        mask_valid      => mask_valid,

        ce_valid        => ce_valid,
        ce_read         => ce_read,

        bpgm_mgf_start  => bpgm_mgf_start,
        bpgm_mgf_done   => bpgm_mgf_done,
        bpgm_mgf_sel    => bpgm_mgf_sel,
        bpgm_mgf_rst    => bpgm_mgf_rst,
        chk_failed_bwc1 => chk_failed_bwc1,
        chk_failed_bwc2 => chk_failed_bwc2,
        msg_len         => msg_len,
        bpgm_mgf_Rferd  => bpgm_mgf_Rferd,

        decrypt         => decrypt,
        sel_param       => sel_param,
        check_3         => check_3,

        --check_1_valid   => check_1_valid,
        check_1         => check_1,
        check1_rst      => check1_rst,
        check1_en_reg   => check1_en_reg,
        check1_last     => check1_last,

        poly_mult_rst   => poly_mult_rst,
        poly_mult_update_h  => poly_mult_update_h,
        poly_mult_sdi_ready => poly_mult_sdi_ready,
        poly_mult_start => poly_mult_start,
        poly_mult_done  => poly_mult_done,
        poly_mult_rF_sel=> poly_mult_rF_sel,

        sel_r_F         => sel_r_F,
        sel_m_cm_di     => sel_m_cm_di,
        sel_b_cb_di     => sel_b_cb_di,
        sel_hTrunc_addr => sel_hTrunc_addr,
        sel_odcu        => sel_odcu,
        sel_ci_mpr      => sel_ci_mpr,

        sel_m_cm_addr   => sel_m_cm_addr,
        sel_b_cb_addr   => sel_b_cb_addr,

        --! hTrunc Distributed RAM ports
        hTrunc_we       => hTrunc_we,
        hTrunc_addr_ctrl=> hTrunc_addr_ctrl,
        --! F Distributed RAM ports
        sel_F_addr      => sel_F_addr,
        F_we            => F_we,
        F_addr_ctrl     => F_addr_ctrl,
        --! m_cm  BRAM ports
        m_cm_we         => m_cm_we,
        m_cm_rd_ctrl    => m_cm_rd_ctrl,
        m_cm_addr_ctrl  => m_cm_addr_ctrl,
        --! b_cb Distributed RAM ports
        b_cb_we         => b_cb_we,
        b_cb_addr_ctrl  => b_cb_addr_ctrl,
        --! DFU-M controling signal
        dfum_sel_regoct => dfum_sel_regoct,
        dfum_sel_data   => dfum_sel_data,
        dfum_sel_sipo   => dfum_sel_sipo,
        dfum_en_mreg    => dfum_en_mreg,
        dfum_en_sipo    => dfum_en_sipo,
        --! ODCU-E controling signal
        odcue_di_valid  => odcue_di_valid,
        odcue_di_ready  => odcue_di_ready,
        --! ODCU-D controling signal
        odcud_en_piso   => odcud_en_piso,
        odcud_ld_piso   => odcud_ld_piso,
        odcud_en_mreg   => odcud_en_mreg,
        odcud_sel_cm    => odcud_sel_cm,
        odcud_sel_cb    => odcud_sel_cb,
        odcud_coctL     => odcud_coctL,
        odcud_check2    => odcud_check2,
        --! IDCU-ED controling signal
        idcued_en_sipo_he   => idcued_en_sipo_he,
        idcued_en_sipo_hram => idcued_en_sipo_hram,
        idcued_en_piso_hram => idcued_en_piso_hram,
        idcued_ld_piso_hram => idcued_ld_piso_hram,
        --! do bus
        do_valid        => do_valid,
        do_ready        => do_ready
    );



    top_controller_inst: entity work.top_controller(behavioral)
    port map(
        clk       => clk,
        rst       => rst,

        pdi_data  => pdi_data,
        pdi_valid => pdi_valid,
        pdi_ready => pdi_ready,

        sdi_data  => sdi_data,
        sdi_valid => sdi_valid,
        sdi_ready => sdi_ready,

        rdi_valid => rdi_valid,
        rdi_ready => rdi_ready,

        ctrl_dout => ctrl_dout,
        sel_ctrl_header => sel_ctrl_header,
        ctrl_do_valid   => ctrl_do_valid,
        ctrl_do_ready   => ctrl_do_ready,

        R_fe_valid_to_ctrl => R_fe_valid_to_ctrl,
        Rfe_rd_from_ctrl   => Rfe_rd_from_ctrl,
        sel_Rfe_rd         => sel_Rfe_rd,

        he_fifo_read    => he_fifo_read,
        he_fifo_empty   => he_fifo_empty,

        mask_ready      => mask_ready,
        mask_valid      => mask_valid,

        ce_valid        => ce_valid,
        ce_read         => ce_read,

        bpgm_mgf_start  => bpgm_mgf_start,
        bpgm_mgf_done   => bpgm_mgf_done,
        bpgm_mgf_sel    => bpgm_mgf_sel,
        bpgm_mgf_rst    => bpgm_mgf_rst,
        chk_failed_bwc1 => chk_failed_bwc1,
        chk_failed_bwc2 => chk_failed_bwc2,
        msg_len         => msg_len,
        bpgm_mgf_Rferd  => bpgm_mgf_Rferd,

        decrypt         => decrypt,
        sel_param       => sel_param,
        check_3         => check_3,

        --check_1_valid   => check_1_valid,
        check_1         => check_1,
        check1_rst      => check1_rst,
        check1_en_reg   => check1_en_reg,
        check1_last     => check1_last,

        poly_mult_rst   => poly_mult_rst,
        poly_mult_update_h  => poly_mult_update_h,
        poly_mult_sdi_ready => poly_mult_sdi_ready,
        poly_mult_start => poly_mult_start,
        poly_mult_done  => poly_mult_done,
        poly_mult_rF_sel=> poly_mult_rF_sel,

        sel_r_F         => sel_r_F,
        sel_m_cm_di     => sel_m_cm_di,
        sel_b_cb_di     => sel_b_cb_di,
        sel_hTrunc_addr => sel_hTrunc_addr,
        sel_odcu        => sel_odcu,
        sel_ci_mpr      => sel_ci_mpr,

        sel_m_cm_addr   => sel_m_cm_addr,
        sel_b_cb_addr   => sel_b_cb_addr,

        --! hTrunc Distributed RAM ports
        hTrunc_we       => hTrunc_we,
        hTrunc_addr_ctrl=> hTrunc_addr_ctrl,
        --! F Distributed RAM ports
        sel_F_addr      => sel_F_addr,
        F_we            => F_we,
        F_addr_ctrl     => F_addr_ctrl,
        --! m_cm  BRAM ports
        m_cm_we         => m_cm_we,
        m_cm_rd_ctrl    => m_cm_rd_ctrl,
        m_cm_addr_ctrl  => m_cm_addr_ctrl,
        --! b_cb Distributed RAM ports
        b_cb_we         => b_cb_we,
        b_cb_addr_ctrl  => b_cb_addr_ctrl,
        --! DFU-M controling signal
        dfum_sel_regoct => dfum_sel_regoct,
        dfum_sel_data   => dfum_sel_data,
        dfum_sel_sipo   => dfum_sel_sipo,
        dfum_en_mreg    => dfum_en_mreg,
        dfum_en_sipo    => dfum_en_sipo,
        --! ODCU-E controling signal
        odcue_di_valid  => odcue_di_valid,
        odcue_di_ready  => odcue_di_ready,
        --! ODCU-D controling signal
        odcud_en_piso   => odcud_en_piso,
        odcud_ld_piso   => odcud_ld_piso,
        odcud_en_mreg   => odcud_en_mreg,
        odcud_sel_cm    => odcud_sel_cm,
        odcud_sel_cb    => odcud_sel_cb,
        odcud_coctL     => odcud_coctL,
        odcud_check2    => odcud_check2,
        --! IDCU-ED controling signal
        idcued_en_sipo_he   => idcued_en_sipo_he,
        idcued_en_sipo_hram => idcued_en_sipo_hram,
        idcued_en_piso_hram => idcued_en_piso_hram,
        idcued_ld_piso_hram => idcued_ld_piso_hram,

        --! Debug
        error  => error

    );


end structure;
