-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================

library ieee;
use ieee.std_logic_1164.all;

entity ntru_top_wrapper is
    generic (
        G_W             : integer := 32;
        G_SW            : integer := 16;
		G_RW            : integer := 32
    );
    port (
        --! Global signals
        clk             : in  std_logic;
        rst             : in  std_logic;

        --! SERDES signals
        sin             : in  std_logic;
        ssel            : in  std_logic;
        sout            : out std_logic
    );
end entity ntru_top_wrapper;

architecture structure of ntru_top_wrapper is
    signal sipo         : std_logic_vector(G_W+G_SW+G_RW+4  -1 downto 0);
    signal piso         : std_logic_vector(G_W+4            -1 downto 0);
    signal piso_data    : std_logic_vector(G_W+4            -1 downto 0);
begin
    process(clk)
    begin
        if rising_edge(clk) then
            sipo <= sin & sipo(sipo'high downto 1);
            if (ssel = '1') then
                piso <= piso_data;
            else
                piso <= '0' & piso(piso'high downto 1);
            end if;
        end if;
    end process;
    sout <= piso(0);

    u_ntru:
    entity work.ntru_top(structure)
        port map (
            clk                     => clk                          ,
            rst                     => rst                          ,

            --! Input signals
            pdi_data                => sipo(       G_W-1 downto   0),
            sdi_data                => sipo(  G_SW+G_W-1 downto G_W),
    		rdi_data                => sipo(  G_RW+G_SW+G_W-1 downto G_SW+G_W),
            pdi_valid               => sipo(1+G_SW+G_W+G_RW-1)           ,
            sdi_valid               => sipo(2+G_SW+G_W+G_RW-1)           ,
    		rdi_valid               => sipo(3+G_SW+G_W+G_RW-1)           ,
            do_ready                => sipo(4+G_SW+G_W+G_RW-1)           ,

            --! Output signals
            do_data                 => piso_data(  G_W-1 downto   0),
            pdi_ready               => piso_data(1+G_W-1)           ,
            sdi_ready               => piso_data(2+G_W-1)           ,
    		rdi_ready               => piso_data(3+G_W-1)           ,
            do_valid                => piso_data(4+G_W-1)
        );
end structure;
