-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Kevin Briggs, Farnoud Farahmand
-- =============================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;

-- Output Data Conversion Unit for Encryption
entity odcu_d is
generic
(
	iw : integer := 96;
	ow : integer := 32
);
port
(
    clk       : in std_logic;
    rst       : in std_logic;

    en_piso   : in std_logic;
    ld_piso   : in std_logic;
    en_mreg   : in std_logic;

    sel_cm    : in std_logic;
    sel_cb    : in std_logic;

    cMbin     : in std_logic_vector(iw-1 downto 0);
	cm        : out std_logic_vector(ow-1 downto 0);
	cb        : out std_logic_vector(ow-1 downto 0);
	coctL     : out std_logic_vector(7 downto 0);
    check_2   : out std_logic_vector(3 downto 0)
);
end odcu_d;

-- ==========================================================================
architecture arch of odcu_d is

    constant zeros    : std_logic_vector(23 downto 0) := (others => '0');

    signal mBin_chunk : std_logic_vector(ow-1 downto 0);
    signal cm_chunk   : std_logic_vector(ow-1 downto 0);
    signal cm_byte    : std_logic_vector(7 downto 0);

    signal mreg_in    : std_logic_vector(23 downto 0);
    signal mreg_out    : std_logic_vector(23 downto 0);
begin

	piso_1 : entity work.piso generic map ( M => ow, N => iw)
		port map (clk => clk, rst => rst, ld => ld_piso, en => en_piso, input => cMbin, output => mBin_chunk );

    mreg : entity work.regn generic map ( N => 24, init => zeros)
            port map ( clk => clk, rst => rst, en => en_mreg, input => mreg_in, output => mreg_out);

    with sel_cm select cm_chunk <=
        mBin_chunk when '0',
        (others => '0') when others;

    with sel_cb select cb <=
        mBin_chunk when '0',
        (others => '0') when others;

    cm_byte <= cm_chunk(31 downto 24);
    mreg_in <= cm_chunk(23 downto 0);

	cm <= mreg_out & cm_byte;

    coctL <= mBin_chunk(31 downto 24);

    Gen_check_2: for i in 0 to 3 generate
        check_2(i) <= '1' when mBin_chunk(8*(i+1)-1 downto 8*i) = X"00" else '0';
    end generate;


end arch;

-- ==========================================================================
