--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;
use work.sha3_pkg.all;
use work.sha2_pkg.all;

entity poly_sub_1 is
	generic
    (
        w : integer := 32;
        ADDR_WIDTH : integer := 4;
        DATA_WIDTH : integer := 2
    );
	port
	(
		a   : in std_logic_vector(2*2*w-1 downto 0);
		b   : in std_logic_vector(2*2*w-1 downto 0);
		r   : out  std_logic_vector(2*2*w-1 downto 0)
	);
end poly_sub_1;

architecture arch of poly_sub_1 is

	type rom_modp_array is array (0 to 2**ADDR_WIDTH-1) of std_logic_vector(DATA_WIDTH-1 downto 0);
    constant rom_modp : rom_modp_array :=
    ("00", "10", "01", "00", "01", "00", "10", "00", "10", "01", "00", "00", "00", "00", "00", "00" );

	type abcr_array is array (0 to 2*w-1) of std_logic_vector(1 downto 0);
	signal ai, bi, ri, rom_out : abcr_array;

	type addr_array is array (0 to 2*w-1) of std_logic_vector(3 downto 0);
	signal addr : addr_array;

begin

	ii : for i in 0 to 2*w-1 generate
		ai(i) <= a(2*(i+1)-1 downto 2*i);
		bi(i) <= b(2*(i+1)-1 downto 2*i);
		addr(i) <= ai(i) & bi(i);
		ri(i) <= rom_modp(to_integer(unsigned (addr(i))));
		r(2*(i+1)-1 downto 2*i) <= ri(i);
	end generate;
end arch;

--=========================================================
