library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.ntru_pkg.all;

entity polymult_wrapper is
  Port ( 
         clk  : in std_logic;
         rst  : in std_logic;
         sin  : in std_logic;
         ssel : in std_logic;
         sout : out std_logic
  );
end polymult_wrapper;

architecture Behavioral of polymult_wrapper is
    signal r_F_sel_sig       : std_logic;
    signal sdi_addr_sig      : std_logic_vector(7 downto 0);
    signal sdi_ready_sig     : std_logic;
    signal i_fifo_empty_sig  : std_logic;
    signal i_fifo_read_sig   : std_logic;
    signal he_fifo_empty_sig : std_logic;
    signal he_fifo_read_sig  : std_logic;
    signal ce_valid_sig      : std_logic;
    signal ce_read_sig       : std_logic;
    signal Rfe_valid_sig     : std_logic;
    signal Rfe_read_sig      : std_logic;
    signal param_sig         : std_logic;
    signal update_h_sig      : std_logic;
    signal decrypt_sig       : std_logic;
    signal start_sig         : std_logic;
    signal done_sig          : std_logic;
    signal r_F_sig           : std_logic_vector(BETA-1 downto 0);
    signal h_e_sig           : std_logic_vector(ALPHA*(W/ALPHA)-1 downto 0);
    signal ce_sig            : std_logic_vector(2*ALPHA*W-1 downto 0);
    signal R_fe_sig          : std_logic_vector(2*ALPHA*W-1 downto 0);
    
    constant N : integer := 1499;
    constant ALPHA : integer := 11;
    constant BETA  : integer := log2ceil(N);
    constant W     : integer := 64;
    constant DF    : integer := 79;

begin

   poly_mult_top : entity work.poly_mult_top (arch) 
   generic map(
       N => N,
       ALPHA => ALPHA,
       BETA => BETA,
       W => W,
       DF => DF,
       PIPE_N => "00000000000",
       PIPE_STAGES => 0
   )
   port map (
       clk => clk,
       rst => rst,
       r_F_sel       => r_F_sel_sig,
       sdi_addr      => sdi_addr_sig,
       sdi_ready     => sdi_ready_sig,
       i_fifo_empty  => i_fifo_empty_sig,
       i_fifo_read   => i_fifo_read_sig,
       he_fifo_empty => he_fifo_empty_sig,
       he_fifo_read  => he_fifo_read_sig,
       ce_valid      => ce_valid_sig,
       ce_read       => ce_read_sig,
       Rfe_valid     => Rfe_valid_sig,
       Rfe_read      => Rfe_read_sig,
       param         => param_sig,
       update_h      => update_h_sig,
       decrypt       => decrypt_sig,
       start         => start_sig,
       done          => done_sig,
       r_F           => r_F_sig,
       h_e           => h_e_sig,
       ce            => ce_sig,
       R_fe          => R_fe_sig
   );

end Behavioral;
