-- =====================================================================
-- Copyright  2011-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- author: Marcin Rogawski
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;  
use ieee.std_logic_unsigned.all; 

-- serial input parallel output

entity range_conv_mod_p is
	generic
	(
		ALPHA :integer := 11;
		w : integer := 32
	);
	port 
	(
		rc_full  : in std_logic_vector(ALPHA*2*w-1 downto 0);
		ci_full : out std_logic_vector(2*2*w-1 downto 0)
	);
end range_conv_mod_p;

architecture arch of range_conv_mod_p is

	type rc_in_array is array (0 to 63) of std_logic_vector(ALPHA-1 downto 0);
	type a_array is array (0 to 63) of std_logic_vector(ALPHA-1 downto 0);
	type even_array is array (0 to 63) of std_logic_vector(3 downto 0);
	type off_array is array (0 to 63) of std_logic_vector(2 downto 0);
	type odd_x2_array is array (0 to 63) of std_logic_vector(3 downto 0);
	type add_array is array (0 to 63) of std_logic_vector(4 downto 0);
	type ci_array is array (0 to 63) of std_logic_vector(1 downto 0);
	
	signal rc_in : rc_in_array;
	signal a : a_array;
	signal even : even_array;
	signal odd : off_array;
	signal odd_x2 : odd_x2_array;
	signal add :add_array;
	signal ci : ci_array;

begin

	ii : for i in 0 to 63 generate
		rc_in(i) <= rc_full(ALPHA*(i+1)-1 downto ALPHA*i);
		a(i) <=  (rc_in(i) - 2)  when (rc_in(i)(ALPHA-1) = '1') else rc_in(i);
		even(i) <= ("000" & a(i)(0)) + a(i)(2) + a(i)(4) + a(i)(6) + a(i)(8) + a(i)(10);
		odd(i)  <= ("00" & a(i)(1)) + a(i)(3) + a(i)(5) + a(i)(7) + a(i)(9);
		odd_x2(i) <= odd(i) & '0';
		
		add(i) <= ('0' & even(i)) + odd_x2(i);
		
		rom_5x2 : entity work.rom_5x2 port map ( addr => add(i), dout => ci(i));
		ci_full(2*(i+1)-1 downto 2*i) <= ci(i); 
	end generate;
end arch;

