--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity t2b is
	generic ( w	: integer := 32);
	port
	(
		a   : in std_logic_vector(4*w-1 downto 0);
		r   : out std_logic_vector(3*w-1 downto 0)
	);
end t2b;

architecture arch of t2b is

	-- arrays
	type ai_array is array(0 to 31) of std_logic_vector(3 downto 0);
	signal ai : ai_array;

	type t2b_rom is array(0 to 15) of std_logic_vector(2 downto 0);

	-- constants
	constant t2b : t2b_rom :=
	(
		0  => "000",
		1  => "001",
		2  => "010",
		3  => "000",
		4  => "011",
		5  => "100",
		6  => "101",
		7  => "000",
		8  => "110",
		9  => "111",
		10 => "000",
		11 => "000",
		12 => "000",
		13 => "000",
		14 => "000",
		15 => "000"
	);

begin

	ii:  for i in 0 to w-1 generate
		ai(i) <= a((4*i)+3 downto 4*i);
		r(3*(i+1)-1 downto 3*i) <= t2b(conv_integer(ai(i)));
	end generate;

end arch;
