--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity var_shiftr is	-- variable shifter (vs)
generic
(
    N : integer := 28
);
port
(
	a : in std_logic_vector(N-1 downto 0);
	b : in std_logic_vector(3 downto 0);
	c : out std_logic_vector(N-1 downto 0)
);
end var_shiftr;

-- ==========================================================================
architecture vsr of var_shiftr is 	-- architecture for variable shift right (vsr) (max shift amount 16)

    constant ZERO : std_logic_vector(N-1 downto 0) := (others => '0');

    type Al_array is array (0 to 4) of std_logic_vector(N-1 downto 0);
    signal Al : Al_array;

    type Ar_array is array (0 to 3) of std_logic_vector(N-1 downto 0);
    signal Ar : Ar_array;

begin
    Al(0) <= a;
    Gen_shft_mux:  for i in 0 to 3 generate
        Ar(i) <=  ZERO(2**i-1 downto 0) & Al(i)(N-1 downto 2**i);
        Al(i+1) <= Al(i) when (b(i) = '0') else Ar(i);
    end generate;

    c <= Al(4);
end vsr;
