-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;

entity bpgm_mgf_poly_mult_tb is

end bpgm_mgf_poly_mult_tb;

architecture behavioral of bpgm_mgf_poly_mult_tb is
constant period : time:= 8 ns;
constant W : integer:=32;
constant BETA : integer:=11;
signal clk: std_logic:='0';
signal rst: std_logic;

signal rst_bpgm_mgf: std_logic;

signal start     : std_logic := '0';
signal done      : std_logic := '0';
signal chk_failed_bwc1 : std_logic := '0';
signal chk_failed_bwc2 : std_logic := '0';
signal decrypt   : std_logic := '0';
signal sel_param : std_logic := '0';
signal sel_bpgm_mgf : std_logic := '0';
signal msg_len    : std_logic_vector(7 downto 0);

signal pdi_di    : std_logic_vector(W - 1 downto 0);
signal pdi_we    : std_logic := '0';

signal pdi_data  : std_logic_vector(W - 1 downto 0);
signal pdi_addr  : std_logic_vector(5 downto 0);
signal pdi_addr_s: std_logic_vector(5 downto 0);
signal pdi_addr_tb: std_logic_vector(5 downto 0);
signal pdi_rd    : std_logic := '0';

signal b_di    : std_logic_vector(W - 1 downto 0);
signal b_we    : std_logic := '0';

signal b       : std_logic_vector(W - 1 downto 0);
signal b_addr  : std_logic_vector(2 downto 0);
signal b_addr_s: std_logic_vector(2 downto 0);
signal b_addr_tb: std_logic_vector(2 downto 0);
signal b_rd    : std_logic := '0';

signal h_di    : std_logic_vector(W - 1 downto 0);
signal h_we    : std_logic := '0';

signal htrunc  : std_logic_vector(W - 1 downto 0);
signal h_addr  : std_logic_vector(2 downto 0);
signal h_addr_s: std_logic_vector(2 downto 0);
signal h_addr_tb: std_logic_vector(2 downto 0);
signal h_rd    : std_logic := '0';

signal cm      : std_logic_vector(W - 1 downto 0);
signal cb      : std_logic_vector(W - 1 downto 0);

signal R4      : std_logic_vector(128 - 1 downto 0);
signal cR4     : std_logic_vector(128 - 1 downto 0);

signal R_fe_valid : std_logic ;
signal R_fe_read  : std_logic;

signal bpgm_fifo_dout   : std_logic_vector(BETA - 1 downto 0);
signal bpgm_fifo_empty  : std_logic := '0';
signal bpgm_fifo_read   : std_logic := '0';
signal bpgm_fifo_almost_full   : std_logic := '0';

signal mask_data   : std_logic_vector(2*2*W - 1 downto 0);
signal mask_valid  : std_logic := '0';
signal mask_ready  : std_logic := '0';

signal sel_addr : std_logic;


signal t_r_F_sel       : std_logic  := '0';
signal t_sdi_addr      : std_logic_vector(log2ceil(2*DF)-1 downto 0);
signal t_sdi_ready     : std_logic  := '1';
signal t_i_fifo_empty  : std_logic  := '0';
signal t_i_fifo_read   : std_logic; -- output
signal t_he_fifo_empty : std_logic  := '1';
signal t_he_fifo_read  : std_logic; -- output
signal t_ce_valid      : std_logic; -- output
signal t_ce_read       : std_logic := '0';
signal t_Rfe_valid     : std_logic; -- output
signal t_Rfe_read      : std_logic := '0';
signal t_param         : std_logic := '0';
signal t_update_h      : std_logic := '0';
signal t_decrypt       : std_logic := '0';
signal t_start         : std_logic := '0';
signal t_done          : std_logic; -- output
signal t_r_F           : std_logic_vector(BETA-1 downto 0);
signal t_h_e           : std_logic_vector((ALPHA*(W/ALPHA))-1 downto 0);
signal t_ce            : std_logic_vector(2*W*ALPHA-1 downto 0);
signal t_R_fe          : std_logic_vector(2*W*ALPHA-1 downto 0) := (others => '0');

begin

clk <= not clk after period/2;


bpgm_mgf_inst : entity work.bpgm_mgf
    port map(
        clk           => clk,
        rst           => rst_bpgm_mgf,

        pdi_data   => pdi_data,
        pdi_addr   => pdi_addr,
        pdi_rd     => pdi_rd,

        b          => b,
        b_addr     => b_addr,
        b_rd       => b_rd,

        htrunc     => htrunc,
        h_addr     => h_addr,
        h_rd       => h_rd,

        cm         => cm,
        cb         => cb,

        R4         => R4,
        cR4        => cR4,

        R_fe_valid => R_fe_valid,
        R_fe_read  => R_fe_read,

        bpgm_fifo_dout  => bpgm_fifo_dout,
        bpgm_fifo_empty => bpgm_fifo_empty,
        bpgm_fifo_read  => bpgm_fifo_read,
        bpgm_fifo_almost_full => bpgm_fifo_almost_full,

        mask_data   => mask_data,
        mask_valid  => mask_valid,
        mask_ready  => mask_ready,

		start         => start,
        chk_failed_bwc1 => chk_failed_bwc1,
        chk_failed_bwc2 => chk_failed_bwc2,
        done          => done,
		decrypt       => decrypt,
        sel_param     => sel_param,
        sel_bpgm_mgf  => sel_bpgm_mgf,
        msg_len       => msg_len
	);

    poly_mult_top : entity work.poly_mult_top (arch)
    generic map (
        N => PARAM_N,
        ALPHA => 11,
        BETA => 11,
        W    => 32,
        DF   => DF,
        PIPE_N => PIPE_N,
        PIPE_STAGES => PIPE_STAGES
    )
    port map (
       clk => clk,
       rst => rst,
       r_F_sel       => t_r_F_sel,
       sdi_addr      => t_sdi_addr,
       sdi_ready     => t_sdi_ready,
       i_fifo_empty  => bpgm_fifo_empty,
       i_fifo_read   => bpgm_fifo_read,
       he_fifo_empty => t_he_fifo_empty,
       he_fifo_read  => t_he_fifo_read,
       ce_valid      => t_ce_valid,
       ce_read       => t_ce_read,
       Rfe_valid     => R_fe_valid,
       Rfe_read      => R_fe_read,
       param         => t_param,
       update_h      => t_update_h,
       decrypt       => t_decrypt,
       start         => t_start,
       done          => t_done,
       r_F           => bpgm_fifo_dout,
       h_e           => t_h_e,
       ce            => t_ce,
       R_fe          => t_R_fe
   );



PDI_RAM: entity work.rams_sp_rf_rst(syn)
    generic map(
    W => 32,
    addr_W => 6,
    addr_S => 64
    )
    port map(
        clk   => clk,
        en    => pdi_rd,
        we    => pdi_we,
        rst   => rst,
        addr  => pdi_addr_s,
        di    => pdi_di,
        do    => pdi_data
    );

-- Distributed RAM
B_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => b_we,
        a     => b_addr_s,
        di    => b_di,
        do    => b
    );

H_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => h_we,
        a     => h_addr_s,
        di    => h_di,
        do    => htrunc
    );

--R4 <= X"000102030405060708090A0B0C0D0E0F";
--  R4 <= X"DEADBEEF0001030406DEADBEEF000103";

  R4 <= t_R_fe (127 downto 0);

  t_h_e <= "10" & x"ABEEF";

pdi_addr_s <= pdi_addr_tb when sel_addr = '1' else pdi_addr;
b_addr_s   <= b_addr_tb when sel_addr = '1' else b_addr;
h_addr_s   <= h_addr_tb when sel_addr = '1' else h_addr;

test_process: process
begin
    rst <= '1';
    rst_bpgm_mgf <= '1';
    wait for period/4;
	wait for period*4;
	rst <= '0';
    rst_bpgm_mgf <= '0';

    wait for period;

    sel_addr <= '1';

    for i in 0 to 63 loop
        pdi_we   <= '1';
        pdi_di   <=  x"DEADBEEF"; --std_logic_vector(to_unsigned(i, pdi_di'length)); --x"00010304"; -- x"DEADBEEF"; --
        pdi_addr_tb <= std_logic_vector(to_unsigned(i, pdi_addr'length));
        wait for period;
    end loop;
    pdi_we <= '0';

    for i in 0 to 45 loop
        b_we   <= '1';
        b_di   <= x"ABCDABCD";
        b_addr_tb <= std_logic_vector(to_unsigned(i, b_addr'length));

        h_we   <= '1';
        h_di   <= std_logic_vector(to_unsigned(i, h_di'length));
        h_addr_tb <= std_logic_vector(to_unsigned(i, h_addr'length));
        wait for period;
    end loop;
    b_we <= '0';
    h_we <= '0';
    sel_addr <= '0';

    t_he_fifo_empty <= '0';
    t_update_h <= '1';
    t_start    <= '1';
    wait for period;
    t_update_h <= '0';
    t_start    <= '0';

    wait until t_done = '1';

    wait for period/4;
    wait for period*3;

    t_he_fifo_empty <= '1';

    --! BWC1 Test
    sel_param    <= '0';
    sel_bpgm_mgf <= '0';
    decrypt      <= '0';
    msg_len      <= X"F7";

    wait for period* 1326; --1158 --1874
	start <= '1';
    t_start    <= '1';
	wait for period;
	start <= '0';
    t_start    <= '0';

    wait until done = '1';

    wait for period/4;
    wait for period*2;

    t_he_fifo_empty <= '1';

    --! BWC2 Test
    sel_param    <= '1';
    -- R_fe_valid   <= t_Rfe_valid;
    sel_bpgm_mgf <= '1';

    rst_bpgm_mgf <= '1';
    wait for period*2;
    rst_bpgm_mgf <= '0';

    wait for period;
	start <= '1';
    t_start    <= '1';
	wait for period;
	start <= '0';
    t_start    <= '0';


	wait;

end process;


read_proc_BWC_2: process(mask_valid)
begin
    if (mask_valid = '1')then
        mask_ready <= '1';
    else
        mask_ready <= '0';
    end if;
end process;

end behavioral;
