-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity rng_trs_tb is

end rng_trs_tb;

architecture behavioral of rng_trs_tb is
constant period : time:= 8 ns;
constant W : integer:=32;
constant BETA : integer:=11;
signal clk: std_logic:='0';
signal rst: std_logic;

signal start     : std_logic := '0';
signal done      : std_logic := '0';
signal chk_failed_bwc1 : std_logic := '0';
signal chk_failed_bwc2 : std_logic := '0';
signal decrypt   : std_logic := '0';
signal sel_param : std_logic := '0';
signal sel_bpgm_mgf : std_logic := '0';
signal msg_len    : std_logic_vector(7 downto 0);

signal pdi_di    : std_logic_vector(W - 1 downto 0);
signal pdi_we    : std_logic := '0';

signal pdi_data  : std_logic_vector(W - 1 downto 0);
signal pdi_addr  : std_logic_vector(5 downto 0);
signal pdi_addr_s: std_logic_vector(5 downto 0);
signal pdi_addr_tb: std_logic_vector(5 downto 0);
signal pdi_rd    : std_logic := '0';

signal b_di    : std_logic_vector(W - 1 downto 0);
signal b_we    : std_logic := '0';

signal b       : std_logic_vector(W - 1 downto 0);
signal b_addr  : std_logic_vector(2 downto 0);
signal b_addr_s: std_logic_vector(2 downto 0);
signal b_addr_tb: std_logic_vector(2 downto 0);
signal b_rd    : std_logic := '0';

signal h_di    : std_logic_vector(W - 1 downto 0);
signal h_we    : std_logic := '0';

signal htrunc  : std_logic_vector(W - 1 downto 0);
signal h_addr  : std_logic_vector(2 downto 0);
signal h_addr_s: std_logic_vector(2 downto 0);
signal h_addr_tb: std_logic_vector(2 downto 0);
signal h_rd    : std_logic := '0';

signal cm      : std_logic_vector(W - 1 downto 0);
signal cb      : std_logic_vector(W - 1 downto 0);

signal R4      : std_logic_vector(128 - 1 downto 0);
signal cR4     : std_logic_vector(128 - 1 downto 0);

signal R_fe_valid : std_logic := '0';
signal R_fe_read  : std_logic ;

signal bpgm_fifo_dout   : std_logic_vector(BETA - 1 downto 0);
signal bpgm_fifo_empty  : std_logic := '0';
signal bpgm_fifo_read   : std_logic := '0';
signal bpgm_fifo_almost_full   : std_logic := '0';

signal mask_data   : std_logic_vector(2*2*W - 1 downto 0);
signal mask_valid  : std_logic := '0';
signal mask_ready  : std_logic := '0';

signal sel_addr : std_logic;

type memory_type is array (7 downto 0) of std_logic_vector(31 downto 0);
signal random : memory_type := (x"0584FAB2", x"36C0B24F",
x"5489F24F", x"C3454082", x"456AC479", x"D2345123", x"9200824F", x"BC345346");

begin

clk <= not clk after period/2;


bpgm_mgf_inst : entity work.bpgm_mgf
    -- generic map (RW => RW,
	--               FIFO_DEPTH => 9)
    port map(
        clk           => clk,
        rst           => rst,

        pdi_data   => pdi_data,
        pdi_addr   => pdi_addr,
        pdi_rd     => pdi_rd,

        b          => b,
        b_addr     => b_addr,
        b_rd       => b_rd,

        htrunc     => htrunc,
        h_addr     => h_addr,
        h_rd       => h_rd,

        cm         => cm,
        cb         => cb,

        R4         => R4,
        cR4        => cR4,

        R_fe_valid => R_fe_valid,
        R_fe_read  => R_fe_read,

        bpgm_fifo_dout  => bpgm_fifo_dout,
        bpgm_fifo_empty => bpgm_fifo_empty,
        bpgm_fifo_read  => bpgm_fifo_read,
        bpgm_fifo_almost_full => bpgm_fifo_almost_full,

        mask_data   => mask_data,
        mask_valid  => mask_valid,
        mask_ready  => mask_ready,

		start         => start,
        chk_failed_bwc1 => chk_failed_bwc1,
        chk_failed_bwc2 => chk_failed_bwc2,
        done          => done,
		decrypt       => decrypt,
        sel_param     => sel_param,
        sel_bpgm_mgf  => sel_bpgm_mgf,
        msg_len       => msg_len
	);

PDI_RAM: entity work.rams_sp_rf_rst(syn)
    generic map(
    W => 32,
    addr_W => 6,
    addr_S => 64
    )
    port map(
        clk   => clk,
        en    => pdi_rd,
        we    => pdi_we,
        rst   => rst,
        addr  => pdi_addr_s,
        di    => pdi_di,
        do    => pdi_data
    );

-- Distributed RAM
B_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => b_we,
        a     => b_addr_s,
        di    => b_di,
        do    => b
    );

H_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => h_we,
        a     => h_addr_s,
        di    => h_di,
        do    => htrunc
    );

--R4 <= X"000102030405060708090A0B0C0D0E0F";
  R4 <= X"DEADBEEF0001030406DEADBEEF000103";

pdi_addr_s <= pdi_addr_tb when sel_addr = '1' else pdi_addr;
b_addr_s   <= b_addr_tb when sel_addr = '1' else b_addr;
h_addr_s   <= h_addr_tb when sel_addr = '1' else h_addr;

test_process: process
begin
    rst <= '1';
    wait for period/4;
	wait for period*4;
	rst <= '0';

    wait for period;

    sel_addr <= '1';

    for i in 0 to 63 loop
        pdi_we   <= '1';
        pdi_di   <=  std_logic_vector(to_unsigned((i+3), pdi_di'length)) ; --xor random(i) --std_logic_vector(to_unsigned(i, pdi_di'length)); --x"00010304"; -- x"DEADBEEF"; --
        pdi_addr_tb <= std_logic_vector(to_unsigned(i, pdi_addr'length));
        wait for period;
    end loop;
    pdi_we <= '0';

    for i in 0 to 7 loop
        b_we   <= '1';
        b_di   <= random(i);-- (std_logic_vector(to_unsigned((i+13), pdi_di'length)))xor (std_logic_vector(to_unsigned((i+2), pdi_di'length))); --x"ABCDABCD"; --std_logic_vector(to_unsigned((i+13), pdi_di'length)); --x"ABCDABCD";
        b_addr_tb <= std_logic_vector(to_unsigned(i, b_addr'length));

        h_we   <= '1';
        h_di   <= random(i) xor (std_logic_vector(to_unsigned((i+60), pdi_di'length))); --random(i) xor  --std_logic_vector(to_unsigned(i, h_di'length));
        h_addr_tb <= std_logic_vector(to_unsigned(i, h_addr'length));
        wait for period;
    end loop;
    b_we <= '0';
    h_we <= '0';

    sel_addr <= '0';

    --! BWC1 Test
    sel_param    <= '0';
    sel_bpgm_mgf <= '0';
    decrypt      <= '0';
    msg_len      <= X"F7";

    wait for period*1911;
	start <= '1';
	wait for period;
	start <= '0';



	wait;

end process;

read_proc_BWC_1: process(bpgm_fifo_empty)
begin
    if (bpgm_fifo_empty = '0')then
        bpgm_fifo_read <= '1';
    else
        bpgm_fifo_read <= '0';
    end if;
end process;

read_proc_BWC_2: process(mask_valid)
begin
    if (mask_valid = '1')then
        mask_ready <= '1';
    else
        mask_ready <= '0';
    end if;
end process;

end behavioral;
