-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;

use ieee.std_logic_textio.all;

library std;
use std.textio.ALL;


entity ntru_top_tb is
end ntru_top_tb;

architecture behavioral of ntru_top_tb is
constant period : time:= 8 ns;
constant W : integer:=32;
constant BETA : integer:=11;
signal clk: std_logic:='0';
signal rst: std_logic;

signal db_32, maxMsgLen_32, maxMsgLen_Byte, DF : integer;

signal pdi_data  : std_logic_vector(PW - 1 downto 0);
signal pdi_valid : std_logic := '0';
signal pdi_ready : std_logic := '0';

signal sdi_data  : std_logic_vector(SW - 1 downto 0);
signal sdi_valid : std_logic := '0';
signal sdi_ready : std_logic := '0';

signal rdi_data  : std_logic_vector(PW - 1 downto 0);
signal rdi_valid : std_logic := '0';
signal rdi_ready : std_logic := '0';

signal do_data   : std_logic_vector(PW - 1 downto 0);
signal do_valid  : std_logic := '0';
signal do_ready  : std_logic := '0';

signal fpdi_din_valid      : std_logic := '0';
signal fpdi_din_ready      : std_logic;
signal fpdi_din            : std_logic_vector(PW - 1 downto 0);

signal frdi_din_valid      : std_logic := '0';
signal frdi_din_ready      : std_logic;
signal frdi_din            : std_logic_vector(PW - 1 downto 0);

signal fsdi_din_valid      : std_logic := '0';
signal fsdi_din_ready      : std_logic;
signal fsdi_din            : std_logic_vector(SW - 1 downto 0);

signal fdo_dout_valid      : std_logic := '0';
signal fdo_dout_ready      : std_logic;
signal fdo_dout            : std_logic_vector(PW - 1 downto 0);


type memory_type is array (PARAM_N-1 downto 0) of std_logic_vector(PW-1 downto 0);
type memory_type_sw is array (158-1 downto 0) of std_logic_vector(SW-1 downto 0);

file vector_file : text open read_mode is "ntru_1499_tv.txt";
signal h_e_array, m_array, rand_array, c_array : memory_type;
signal F_array : memory_type_sw;

signal done_reading_vectorfile  : std_logic := '0';


begin

clk <= not clk after period/2;

TB_1087_PARAM_GEN: if PARAM_N = 1087 generate
    db_32          <= db_32_1087;
    maxMsgLen_32   <= maxMsgLen_32_1087;
    maxMsgLen_Byte <= maxMsgLen_Byte_1087;
    DF             <= DF_1087;
end generate;

TB_1499_PARAM_GEN: if PARAM_N = 1499 generate
    db_32          <= db_32_1499;
    maxMsgLen_32   <= maxMsgLen_32_1499;
    maxMsgLen_Byte <= maxMsgLen_Byte_1499;
    DF             <= DF_1499;
end generate;

genPDIfifo: entity work.fwft_fifo(structure)
generic map (
    G_ASYNC_RSTN => False,
    G_W          => PW,
    G_LOG2DEPTH  => 5)
port map (
    clk          =>  clk,
    rst          =>  rst,
    din          =>  fpdi_din,
    din_valid    =>  fpdi_din_valid,
    din_ready    =>  fpdi_din_ready,
    dout         =>  pdi_data,
    dout_valid   =>  pdi_valid,
    dout_ready   =>  pdi_ready
);

genRDIfifo: entity work.fwft_fifo(structure)
generic map (
    G_ASYNC_RSTN => False,
    G_W          => PW,
    G_LOG2DEPTH  => 5)
port map (
    clk          =>  clk,
    rst          =>  rst,
    din          =>  frdi_din,
    din_valid    =>  frdi_din_valid,
    din_ready    =>  frdi_din_ready,
    dout         =>  rdi_data,
    dout_valid   =>  rdi_valid,
    dout_ready   =>  rdi_ready
);

genSDIfifo: entity work.fwft_fifo(structure)
generic map (
    G_ASYNC_RSTN => False,
    G_W          => SW,
    G_LOG2DEPTH  => 5)
port map (
    clk          =>  clk,
    rst          =>  rst,
    din          =>  fsdi_din,
    din_valid    =>  fsdi_din_valid,
    din_ready    =>  fsdi_din_ready,
    dout         =>  sdi_data,
    dout_valid   =>  sdi_valid,
    dout_ready   =>  sdi_ready
);

genDOfifo: entity work.fwft_fifo(structure)
generic map (
    G_ASYNC_RSTN => False,
    G_W          => PW,
    G_LOG2DEPTH  => 5)
port map (
    clk          =>  clk,
    rst          =>  rst,
    din          =>  do_data,
    din_valid    =>  do_valid,
    din_ready    =>  do_ready,
    dout         =>  fdo_dout,
    dout_valid   =>  fdo_dout_valid,
    dout_ready   =>  fdo_dout_ready
);

ntru_top_inst : entity work.ntru_top(structure)
    -- generic map (RW => RW,
	--               FIFO_DEPTH => 9)
    port map(
        clk           => clk,
        rst           => rst,

        sdi_valid     => sdi_valid,
        sdi_ready     => sdi_ready,
        sdi_data      => sdi_data,

        pdi_valid     => pdi_valid,
        pdi_ready     => pdi_ready,
        pdi_data      => pdi_data,

        rdi_valid     => rdi_valid,
        rdi_ready     => rdi_ready,
        rdi_data      => rdi_data,

        do_valid      => do_valid,
        do_ready      => do_ready,
        do_data       => do_data
	);



test_process: process
begin
    rst <= '1';

	wait for period*10;
	rst <= '0';

    wait for period/2;

    wait for period*2;

    fpdi_din  <= "101" & "00000" & X"00" & X"21F8";
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load Public Key
    report "**** START - LOAD PUBLIC KEY ****";
    for i in 0 to PARAM_N-1 loop
        fpdi_din <= h_e_array(i);
        fpdi_din_valid <= '1';
        if ( fpdi_din_ready = '0' ) then
            fpdi_din_valid <= '0';
            wait until  fpdi_din_ready <= '1';
            wait for    period/2; --! write in the rising edge
            fpdi_din_valid <= '1';
        end if;
        wait for period;
    end loop;
    report "**** END - LOAD PUBLIC KEY ****";


    --! Encryption instruction
    report "**** START - ENCRYPTION ****";
    fpdi_din  <= X"0" & X"0000000";
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load MSG header
    fpdi_din  <= "001" & "00011" & X"00" & std_logic_vector(to_unsigned(maxMsgLen_Byte, 16));
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load MSG and Random data
    report "**** START - LOAD MESSAGE and RANDOM NUMBER ****";
    for i in 0 to maxMsgLen_32-1 loop
        fpdi_din  <= m_array(i);
        fpdi_din_valid <= '1';
        if ( fpdi_din_ready = '0' ) then
            fpdi_din_valid <= '0';
            wait until  fpdi_din_ready <= '1';
            wait for    period/2; --! write in the rising edge
            fpdi_din_valid <= '1';
        end if;
        if (i < db_32) then
            frdi_din  <= rand_array(i);
            frdi_din_valid <= '1';
            if ( frdi_din_ready = '0' ) then
                frdi_din_valid <= '0';
                fpdi_din_valid <= '0';
                wait until  frdi_din_ready <= '1';
                wait for    period/2; --! write in the rising edge
                frdi_din_valid <= '1';
            end if;
        else
            frdi_din_valid <= '0';
        end if;
        wait for period;
    end loop;
    report "**** END - LOAD MESSAGE and RANDOM NUMBER ****";

    frdi_din_valid <= '0';
    fpdi_din_valid <= '0';

    --! wait for the output data to be ready
    fdo_dout_ready <= '1';
    if ( fdo_dout_valid = '0') then
        fdo_dout_ready <= '0';
        wait until fdo_dout_valid = '1';
        wait for period/2;
        fdo_dout_ready <= '1';
    end if;

    report "**** END - ENCRYPTION ****";
    report "**** START - ENCRYPTION VERIFICATION****";
    report "Ciphertext header:";
    report slv_to_string(do_data);
    wait for period;
    fdo_dout_ready <= '0';

    for i in 0 to PARAM_N-1 loop
        fdo_dout_ready <= '1';
        if ( fdo_dout_valid = '0') then
            fdo_dout_ready <= '0';
            wait until fdo_dout_valid = '1';
            wait for period/2;
            fdo_dout_ready <= '1';
        end if;
        if (fdo_dout /= c_array(i)) then
            report "ERROR:";
            report "Expect:";
            report slv_to_string(c_array(i));
            report "Received:";
            report slv_to_string(do_data);
        end if;
        wait for period;
    end loop;

    fdo_dout_ready <= '1';
    if ( fdo_dout_valid = '0') then
        fdo_dout_ready <= '0';
        wait until fdo_dout_valid = '1';
        wait for period/2;
        fdo_dout_ready <= '1';
    end if;
    if (fdo_dout /= x"E0000000") then
        report "Encryption Status: FAILD";
    else
        report "Encryption Status: PASSED";
    end if;
    wait for period;
    fdo_dout_ready <= '0';

    report "**** END - ENCRYPTION VERIFICATION****";

    wait for period*10;

    --! Load Private Key Instruction
    fpdi_din  <= X"6" & X"0000000";
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;
    fpdi_din_valid <= '0';

    --! Load Private Key header
    fsdi_din  <= "110" & "00000" & X"7E";
    fsdi_din_valid <= '1';
    if ( fsdi_din_ready = '0' ) then
        fsdi_din_valid <= '0';
        wait until  fsdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fsdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load Private Key
    report "**** START - LOAD PRIVATE KEY ****";
    for i in 0 to (2*DF)-1 loop
        fsdi_din <= F_array(i);
        fsdi_din_valid <= '1';
        if ( fsdi_din_ready = '0' ) then
            fsdi_din_valid <= '0';
            wait until  fsdi_din_ready <= '1';
            wait for    period/2; --! write in the rising edge
            fsdi_din_valid <= '1';
        end if;
        wait for period;
    end loop;
    report "**** END - LOAD PRIVATE KEY ****";

    --! Decryption Instruction
    report "**** START - DECRYPTION ****";
    fpdi_din  <= X"1" & X"0000000";
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load Ciphertext header
    fpdi_din  <= "011" & "00011" & X"00" & std_logic_vector(to_unsigned(PARAM_N*4, 16));
    fpdi_din_valid <= '1';
    if ( fpdi_din_ready = '0' ) then
        fpdi_din_valid <= '0';
        wait until  fpdi_din_ready <= '1';
        wait for    period/2; --! write in the rising edge
        fpdi_din_valid <= '1';
    end if;
    wait for period;

    --! Load Ciphertext
    report "**** START - LOAD CIPHERTEXT ****";
    for i in 0 to PARAM_N-1 loop
        fpdi_din <= c_array(i);
        fpdi_din_valid <= '1';
        if ( fpdi_din_ready = '0' ) then
            fpdi_din_valid <= '0';
            wait until  fpdi_din_ready <= '1';
            wait for    period/2; --! write in the rising edge
            fpdi_din_valid <= '1';
        end if;
        wait for period;
    end loop;
    report "**** END - LOAD CIPHERTEXT ****";

    --! wait for the output data to be ready


    fdo_dout_ready <= '1';
    if ( fdo_dout_valid = '0') then
        fdo_dout_ready <= '0';
        wait until fdo_dout_valid = '1';
        wait for period/2;
        fdo_dout_ready <= '1';
    end if;

    report "**** END - DECRYPTION ****";
    report "**** START - DECRYPTION VERIFICATION****";
    report "Message header:";
    report slv_to_string(do_data);
    wait for period;
    fdo_dout_ready <= '0';

    for i in 0 to maxMsgLen_32-1 loop
        fdo_dout_ready <= '1';
        if ( fdo_dout_valid = '0') then
            fdo_dout_ready <= '0';
            wait until fdo_dout_valid = '1';
            wait for period/2;
            fdo_dout_ready <= '1';
        end if;
        if (fdo_dout /= m_array(i)) then
            report "ERROR:";
            report "Expect:";
            report slv_to_string(m_array(i));
            report "Received:";
            report slv_to_string(do_data);
        end if;
        wait for period;
    end loop;

    fdo_dout_ready <= '1';
    if ( fdo_dout_valid = '0') then
        fdo_dout_ready <= '0';
        wait until fdo_dout_valid = '1';
        wait for period/2;
        fdo_dout_ready <= '1';
    end if;
    if (fdo_dout /= x"E0000000") then
        report "Decryption Status: FAILD";
    else
        report "Decryption Status: PASSED";
    end if;
    wait for period;
    fdo_dout_ready <= '0';
    report "**** END - DECRYPTION VERIFICATION****";

	wait;
end process;

test_vector_input : process
    variable vector_line   : line;
    variable v_h_e         : std_logic_vector(PW - 1 downto 0);
    variable v_m           : std_logic_vector(PW - 1 downto 0);
    variable v_rand        : std_logic_vector(PW - 1 downto 0);
    variable v_c           : std_logic_vector(PW - 1 downto 0);
    variable v_F           : std_logic_vector(SW - 1 downto 0);
    variable linecount     : integer := 0;
    variable text          : string(1 to 9);
begin
    wait for period*2;
    report "Start reading the input file...";
    while not endfile(vector_file) loop
        -- report "read line #" & integer'image(linecount);
        readline(vector_file, vector_line);

        -- read N lines of coefficients of h
        if (linecount = 0) then
            report "START reading Test Vectors ...";

        elsif (linecount < (PARAM_N+1)) then
            hread(vector_line, v_h_e);
            h_e_array(linecount-1) <= v_h_e;

        elsif (linecount = (PARAM_N+1)) then
            report "DONE reading Public Key ...";

        elsif(linecount < (PARAM_N + maxMsgLen_32 + 2)) then
            hread(vector_line, v_m);
            m_array(linecount - PARAM_N - 2) <= v_m;

        elsif (linecount = (PARAM_N + maxMsgLen_32 + 2)) then

            report "DONE reading Input MSG ...";

        elsif(linecount < (PARAM_N + maxMsgLen_32 + db_32 + 3)) then
            hread(vector_line, v_rand);
            rand_array(linecount - PARAM_N - maxMsgLen_32 - 3) <= v_rand;

        elsif(linecount = (PARAM_N + maxMsgLen_32 + db_32 + 3)) then

            report "DONE reading Random Number ...";

        elsif(linecount < (PARAM_N + maxMsgLen_32 + db_32 + PARAM_N + 4)) then
            hread(vector_line, v_c);
            c_array(linecount - PARAM_N - maxMsgLen_32 - db_32 - 4) <= v_c;

        elsif(linecount = (PARAM_N + maxMsgLen_32 + db_32 + PARAM_N + 4)) then
            report "DONE reading Ciphertext";

        else
            hread(vector_line, v_F);
            F_array(linecount - PARAM_N - maxMsgLen_32 - db_32 - PARAM_N - 5) <= v_F;

        end if;

        linecount := linecount + 1;

    end loop;

    report "DONE reading Private Key F";
    done_reading_vectorfile <= '1';

    wait;
end process;


end behavioral;
